/**
 * vue-stator v0.0.10
 * (c) 2019
 * - Jonas Galvez (@galvez)
 * - Pim (@pimlie)
 * - All the amazing contributors
 * @license MIT
 */

import Vue from 'vue';

function $set() {
  Vue.set.apply(Vue, arguments);
}
function $delete() {
  Vue.delete.apply(Vue, arguments);
}
function mapState(namespace, properties) {
  if (Array.isArray(namespace)) {
    return namespace.reduce(function (mapped, prop) {
      mapped[prop] = {
        get: function get() {
          return this.$state[prop];
        },
        set: function set(value) {
          this.$state[prop] = value;
          return this.$state[prop];
        }
      };
      return mapped;
    }, {});
  }

  return properties.reduce(function (mapped, prop) {
    mapped[prop] = {
      get: function get() {
        return this.$state[namespace][prop];
      },
      set: function set(value) {
        this.$state[namespace][prop] = value;
        return this.$state[prop];
      }
    };
    return mapped;
  }, {});
}
function mapActions(namespace, properties) {
  if (Array.isArray(namespace)) {
    return namespace.reduce(function (mapped, prop) {
      mapped[prop] = function () {
        var _this$$actions;

        return (_this$$actions = this.$actions)[prop].apply(_this$$actions, arguments);
      };

      return mapped;
    }, {});
  }

  return properties.reduce(function (mapped, prop) {
    mapped[prop] = function () {
      var _this$$actions$namesp;

      return (_this$$actions$namesp = this.$actions[namespace])[prop].apply(_this$$actions$namesp, arguments);
    };

    return mapped;
  }, {});
}
function mapGetters(namespace, getters) {
  if (Array.isArray(namespace)) {
    return namespace.reduce(function (mapped, getter) {
      mapped[getter] = function () {
        return this.$getters[getter];
      };

      return mapped;
    }, {});
  }

  return getters.reduce(function (mapped, getter) {
    mapped[getter] = function () {
      return this.$getters[namespace][getter];
    };

    return mapped;
  }, {});
}

// Internal utilities
function dumpState(storage, namespace, data) {
  if (!storage) {
    return;
  }

  storage.setItem("vue-stator:".concat(namespace), JSON.stringify(data));
}
function loadState(storage, namespace, onSuccess, onError) {
  if (!storage) {
    return;
  }

  try {
    var data = JSON.parse(storage.getItem("vue-stator:".concat(namespace)));

    if (data) {
      onSuccess(data);
    }
  } catch (err) {
    // eslint-disable-next-line no-console
    console.error("[vue-stator] error in loadState(".concat(Array.prototype.slice.call(arguments).join(', '), ")"), err);
  }
}

var watcher;
function subscribe(_ref, key, callback) {
  var $state = _ref.$state;

  if (!watcher) {
    watcher = new Vue();
  }

  return watcher.$watch(function () {
    return $state[key];
  }, callback, {
    deep: true
  });
}

function createStore(_ref) {
  var ctx = _ref.ctx,
      state = _ref.state,
      actions = _ref.actions,
      getters = _ref.getters,
      hydrate = _ref.hydrate,
      localStorage = _ref.localStorage,
      sessionStorage = _ref.sessionStorage;
  var initialState = state(ctx);

  if (ctx._isVue) {
    ctx = {};
  }

  if (localStorage) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      var _loop = function _loop() {
        var namespace = _step.value;
        loadState(window.localStorage, state, function (data) {
          return $set(ctx.$state, namespace, data);
        });
        subscribe(ctx, namespace, function (snapshot) {
          return dumpState(window.localStorage, namespace, snapshot);
        });
      };

      for (var _iterator = localStorage[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        _loop();
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  }

  if (sessionStorage) {
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      var _loop2 = function _loop2() {
        var namespace = _step2.value;
        loadState(window.sessionStorage, state, function (data) {
          return $set(ctx.$state, namespace, data);
        });
        subscribe(ctx, namespace, function (snapshot) {
          return dumpState(window.sessionStorage, namespace, snapshot);
        });
      };

      for (var _iterator2 = sessionStorage[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        _loop2();
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }
  }

  if (hydrate) {
    ctx.$state = hydrate(initialState);
  } else {
    ctx.$state = Vue.observable(initialState);
  }

  if (getters) {
    ctx.$getters = {};
    registerGetters(ctx, getters);
  }

  if (actions) {
    ctx.$actions = {};
    registerActions(ctx, actions);
  }

  return ctx;
} // Registration helpers

function registerGetters(ctx, getters) {
  ctx.$getters = {};

  var _loop3 = function _loop3(key) {
    if (typeof getters[key] === 'function') {
      Object.defineProperty(ctx.$getters, key, {
        get: function get() {
          return getters[key](ctx.$state, ctx.$getters);
        }
      });
      return "continue";
    }

    ctx.$getters[key] = {};

    for (var subKey in getters[key]) {
      Object.defineProperty(ctx.$getters[key], subKey, {
        get: function get() {
          return getters[key](ctx.$state[key], ctx.$getters[key], ctx.$state);
        }
      });
    }
  };

  for (var key in getters) {
    var _ret = _loop3(key);

    if (_ret === "continue") continue;
  }

  return ctx.$getters;
}
function registerActions(ctx, actions) {
  ctx.$actions = {};

  var _loop4 = function _loop4(key) {
    if (typeof actions[key] === 'function') {
      ctx.$actions[key] = function () {
        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return actions[key].apply(actions, [ctx, ctx.$state].concat(args));
      };

      return "continue";
    }

    ctx.$actions[key] = {};

    var _loop5 = function _loop5(subKey) {
      ctx.$actions[key][subKey] = function () {
        var _actions$key;

        for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        return (_actions$key = actions[key])[subKey].apply(_actions$key, [ctx, ctx.$state[key]].concat(args));
      };
    };

    for (var subKey in actions[key]) {
      _loop5(subKey);
    }
  };

  for (var key in actions) {
    var _ret2 = _loop4(key);

    if (_ret2 === "continue") continue;
  }

  return ctx.$actions;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(source, true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(source).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function lazyInject(Vue, key, setter) {
  // Check if plugin not already installed
  var installKey = "__stator_".concat(key, "_installed");

  if (Vue[installKey]) {
    return;
  }

  Vue[installKey] = true; // Call Vue.use() to install the plugin into vm

  Vue.use(function () {
    if (!Vue.prototype.hasOwnProperty(key)) {
      Object.defineProperty(Vue.prototype, key, {
        configurable: true,
        get: function get() {
          this.$root.$options[key] = setter(this);
          Object.defineProperty(Vue.prototype, key, {
            get: function get() {
              return this.$root.$options[key];
            }
          });
          return this.$root.$options[key];
        }
      });
    }
  });
}

var plugin = {
  install: function install(Vue, options) {
    lazyInject(Vue, '_stator', function (vm) {
      return createStore(_objectSpread2({
        ctx: vm
      }, options));
    });
    lazyInject(Vue, '$state', function (vm) {
      return vm._stator.$state;
    });
    lazyInject(Vue, '$getters', function (vm) {
      return vm._stator.$getters;
    });
    lazyInject(Vue, '$actions', function (vm) {
      return vm._stator.$actions;
    });
  }
};

export { $delete, $set, createStore, mapActions, mapGetters, mapState, plugin, registerActions, registerGetters, subscribe };
